from re import L
import pygame
import os, sys
from python.player import Player
from python.world import World, MiniMap
from python.hud import HUD
from python.items import Items, Item
from python.chests import Chest
from python.enemies import Enemy, Enemies
from python.npc import NPCS, NPC
from python.particles import Particle, Particles
from python.utils import *
from python.mainmenu import menu


class Game():
    def __init__(self, win):
        self.start_time = 0
        self.dt = 0
        self.win = win
        self.clock = pygame.time.Clock()
        self.width, self.height = self.win.get_size()
        self.show_outline = False
        self.particles = Particles()
        self.player = Player((self.width, self.height), (self.width / 2, self.height / 2), self.particles)
        self.enemies = Enemies()
        #self.enemies.enemies_spawned.append(Enemy(self.width / 2 + 500, self.height / 2, "large mushroom", self.enemies.enemies["large mushroom"], self.enemies.art["large mushroom"], self.particles))
        self.npcs = NPCS()
        self.items = Items()
        self.npcs.npcs_spawned.append(NPC(self.width/2, self.height/2, "merchant", self.npcs.npcs["merchant"], self.npcs.art["merchant"], self.items))
        self.hud = HUD((self.width, self.height))
        self.hud.inv.add_item(Item("healing_potion", self.items.items["healing_potion"], self.items.art["healing_potion"]))
        self.chests = []        
        self.opened_chest = None
        self.interaction = False
        self.world = World((self.width, self.height), self.chests, self.enemies, self.particles, self.items)
        self.chests = self.world.reset_world(self.player, self.enemies, self.chests, self.items, self.particles)
        self.current_world = 1

    def update(self):
        self.start_time += 1 
        self.dt = self.clock.tick(60) * 0.001 * 60
        if not self.hud.inventory_open and self.opened_chest == None and self.interaction == False:
            self.player.update(self.dt, self.enemies)
            self.enemies.update(self.dt, self.player)
            interaction = self.npcs.update(self.dt, self.player, self.world.scroll)
            if interaction != None:
                self.interaction = interaction
        self.world.update(self.player, self.enemies, self.chests, self.items, self.particles)
        self.world.center_camera(self.player)

    def handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.KEYDOWN:
                interaction = self.npcs.handle_keys(event.key)
                if interaction != None:
                    self.interaction = interaction
                if not self.interaction:
                    if event.key == pygame.K_e:
                        if self.opened_chest == None:
                            self.hud.inventory_open = not self.hud.inventory_open
                        else:
                            self.opened_chest.open = False
                            self.opened_chest = None
                    if event.key == pygame.K_ESCAPE:
                        self.hud.inventory_open = False
                        if self.opened_chest != None:
                            self.opened_chest.open = False
                            self.opened_chest = None
                    if event.key == pygame.K_F1:
                        self.show_outline = not self.show_outline
                    if event.key == pygame.K_n:
                        if self.enemies.enemies_spawned == []:
                            self.current_world += 1
                            self.chests = self.world.reset_world(self.player, self.enemies, self.chests, self.items, self.particles)
            if event.type == pygame.MOUSEBUTTONDOWN:
                interaction = self.npcs.handle_click(event.button, self.hud.inv, self.player)
                if interaction != None:
                    self.interaction = interaction
                if not self.interaction:
                    if self.hud.inventory_open:
                        self.hud.inv.on_click(event.button, self.player)
                    else:
                        if self.opened_chest == None:
                            if event.button == 1:
                                if self.player.gun_equiped:
                                    self.player.attack()
                                    self.player.since_gun_equiped = 0
                                else:
                                    self.player.gun_equiped = True
                            elif event.button == 3:
                                self.player.gun_equiped = False
                            for chest in self.chests:
                                if mouse_over(pygame.mouse.get_pos(), chest.x-self.world.scroll.x, chest.y-self.world.scroll.y, chest.rect.width, chest.rect.height):
                                    chest.open = True
                                    self.opened_chest = chest
                        else:
                            self.opened_chest.on_click(event.button, self.hud.inv)

    def render(self):
        self.win.fill((0, 0, 0))
        self.world.render(self.win)
        #self.minimap.render(self.win)
        for chest in self.chests:
            chest.render(self.win, self.world.scroll)
        self.enemies.render(self.win, self.world.scroll, self.dt, self.start_time, self.show_outline)
        self.player.render(self.win, self.world.scroll, self.dt, self.start_time, self.show_outline)
        self.particles.render(self.win, self.world.scroll)
        if self.opened_chest != None:
            self.opened_chest.render_inv(self.win)
        self.npcs.render(self.win, self.world.scroll, self.dt, self.start_time, self.show_outline)
        self.hud.render(self.win, self.player, self.enemies)
        if self.enemies.enemies_spawned == []:
            draw_large_text(self.win, "You have killed all the enemies! Press N to travel to the next planet!",
            (self.width/2-500, self.height/2-250), pygame.font.Font("assets/font.ttf", 32))


def game_loop(win):
    game = Game(win)
    while True:
        if game.player.dead == False:
            game.update()
            game.handle_events()
            game.render()
            pygame.display.update()
        else:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_n:
                        game_loop(win)
                    elif event.key == pygame.K_q:
                        pygame.quit()
                        sys.exit()
            game.win.fill((0, 0, 0))
            draw_text(game.win, (game.width/2-300, game.height/2), "Game Over. Press N to start a new game or Q to exit", 30, (255,255,255))
            draw_text(game.win, (game.width/2-300, game.height/2+100), f"You have passed {game.current_world} planet(s)", 30, (255,255,255))
            pygame.display.update()

if __name__ == "__main__":
    pygame.init()
    pygame.mixer.init()
    win = pygame.display.set_mode((1280, 720))
    pygame.display.set_caption("Astral Red")
    mainmenu = menu()
    while True:
        command = mainmenu.update()
        mainmenu.render(win)
        if command == "start":
            del mainmenu
            game_loop(win)
        elif command == "tutorial":
            mainmenu.in_tutorial = True
