import pygame
from python.utils import *

# lot of the code here is similar to the inventory code from hud.py

INV_EMPTY = (23, 88, 101)
INV_NOT_EMPTY = (46, 209, 239)
INV_SELECTED = (255,215,0)
INV_HOVER = (0, 215, 255)
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)


class Chest():
    def __init__(self, winsize, coords):
        self.win_size = winsize
        self.x = coords[0]
        self.y = coords[1]
        self.rect = pygame.Rect(self.x, self.y, 60, 45)
        self.items = []
        self.image = pygame.transform.scale(pygame.image.load('assets/chest.png').convert_alpha(), (60, 45))
        self.open = False
        self.inv_box_size = (50,50)
        self.next_slot = 1
        self.selected_slot = 0
        self.dragging_item = None
        self.hover_rect = None
        self.rendered_items = []
        self.take_btn = None

    def render(self, win, scroll):
        win.blit(self.image, (self.x - scroll.x, self.y - scroll.y))

    def render_inv(self, win):
        y = 0
        x = 1
        init_x = (self.win_size[0]/2-(self.inv_box_size[0]*10/2))-300
        init_y = self.win_size[1]/3
        self.hover_rect = None
        for z in range(30):
            color = INV_NOT_EMPTY if self.check_slot(z+1) else INV_EMPTY
            rect = pygame.draw.rect(win, color,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]))

            outline_rect = pygame.draw.rect(win, INV_SELECTED if self.selected_slot == z+1 else BLACK,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]), 4)

            self.draw_item(win, z+1, rect)
            if mouse_over(pygame.mouse.get_pos(), outline_rect.left, outline_rect.top, outline_rect.width, outline_rect.height):
                self.hover_rect = (rect, z+1) # This is needed so that it draws the hover box on top of the inventory
                if self.dragging_item != None: # dragging item rect outline
                    outline_rect = pygame.draw.rect(win, INV_HOVER,
                    (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
                    self.inv_box_size[0], self.inv_box_size[1]), 4)

            if str(x).endswith("0") and x != 0:
                y += self.inv_box_size[1]+4.1
                x = 1
            else:
                x += 1
        
        self.render_take_btn(win)
        self.preview_box(win)
        if self.hover_rect != None:
            self.hover_item(self.hover_rect[0], self.hover_rect[1], win)
        
        if self.dragging_item != None:
            img = self.dragging_item["img"].copy()
            img.fill((255, 255, 255, 200), None, pygame.BLEND_RGBA_MULT)
            win.blit(img, (pygame.mouse.get_pos()[0]-img.get_width()/2, pygame.mouse.get_pos()[1]-img.get_height()/2))

    def render_take_btn(self, win):
        x = (self.win_size[0]/2-(self.inv_box_size[0]*10/2))-245
        y = self.win_size[1]/3-59

        font = pygame.font.Font('assets/font.ttf', 32-len("Take"))
        info_box = pygame.Rect(x, y, 65, 32)
        txt_surface = font.render("Take", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED, info_box, 7) 
        self.take_btn = rect

    def preview_box(self, win):
        item = self.check_slot(self.selected_slot)
        rect = pygame.draw.rect(win, BLACK,
        (int(self.win_size[0]-500), int(self.win_size[1]/3), 
        350, 400))

        if item != False:
            w = 200
            h = 200
            win.blit(pygame.transform.scale(item["img"], (w, h)), (rect.center[0]-w/2+15, rect.center[1]-h/1.5))    
            draw_text(win, (rect.center[0]-w/2+15, rect.center[1]-h/1.5-24), item["item"].name, 24, WHITE, "assets/font.ttf")
            #draw_text(win, (rect.left+20, rect.center[1]+40), item["item"].item["description"], 20, WHITE, "assets/font.ttf")
            font = pygame.font.Font("assets/font.ttf", 20)
            draw_large_text(win, item["item"].item["description"], (rect.left+20, rect.center[1]+40), font, WHITE)
            draw_text(win, (rect.left+20, rect.center[1]+80), "Type: " + item["item"].item["type"], 20, WHITE, "assets/font.ttf")

            item_dict = item["item"].item
            x, y = rect.left+20, rect.center[1]+100
            for effect in item_dict["effects"]:
                draw_text(win, (x, y), f"{effect.capitalize()}: " + str(item_dict[effect]), 20, WHITE, "assets/font.ttf")
                y += 20

    def on_click(self, btn, inventory):
        try:
            if btn == 1: # Left click
                if self.take_btn != None and self.check_slot(self.selected_slot) != False:
                    if mouse_over(pygame.mouse.get_pos(), self.take_btn.left, self.take_btn.top, self.take_btn.width, self.take_btn.height):
                        inventory.add_item(self.check_slot(self.selected_slot)["item"])
                        self.check_slot(self.selected_slot)["amount"] -= 1
                        if self.check_slot(self.selected_slot)["amount"] <= 0:
                            self.items.remove(self.check_slot(self.selected_slot))

                # Dragging item
                if self.dragging_item == None and self.selected_slot != 0 and self.selected_slot != None and self.check_slot(self.selected_slot) != False:
                    if mouse_over(pygame.mouse.get_pos(), self.hover_rect[0].left, self.hover_rect[0].top, self.hover_rect[0].width+4, self.hover_rect[0].height+4):
                        selected_slot = self.hover_rect[1]
                    else:
                        selected_slot = None
                    if self.selected_slot == selected_slot:
                        s = self.check_slot(self.selected_slot)
                        self.dragging_item = s if s is not False else None
                # Dropping item
                elif self.dragging_item != None and self.hover_rect != None:
                    is_origin_slot = bool(self.hover_rect[1] == self.dragging_item["slot"])
                    if not is_origin_slot:
                        current_slot = self.check_slot(self.hover_rect[1])
                        if current_slot != False:
                            if current_slot["item"].id == self.dragging_item["item"].id:
                                current_slot["amount"] += self.dragging_item["amount"]
                                self.items.remove(self.dragging_item)
                                self.dragging_item = None
                            else:
                                self.dragging_item["slot"] = self.hover_rect[1]
                                self.dragging_item = current_slot
                                current_slot["slot"] = 0 # 0 means it is not in the inventory
                        else:
                            self.dragging_item["slot"] = self.hover_rect[1]
                            self.dragging_item = None
                        self.selected_slot = self.hover_rect[1]
                    elif is_origin_slot:
                        self.dragging_item = None
                # Selecting a slot
                if self.hover_rect != None:
                    if mouse_over(pygame.mouse.get_pos(), self.hover_rect[0].left, self.hover_rect[0].top, self.hover_rect[0].width+4, self.hover_rect[0].height+4):
                        self.selected_slot = self.hover_rect[1]

            elif btn == 2: # Middle click
                pass
                                
            elif btn == 3: # Right click
                if self.dragging_item != None and self.hover_rect != None: # dropping only 1 item from stack
                    is_origin_slot = bool(self.hover_rect[1] == self.dragging_item["slot"])
                    if not is_origin_slot:
                        current_slot = self.check_slot(self.hover_rect[1])
                        if current_slot != False:
                            if current_slot["item"].id == self.dragging_item["item"].id:
                                current_slot["amount"] += 1
                                self.dragging_item["amount"] -= 1
                                if self.dragging_item["amount"] == 0:
                                    self.items.remove(self.dragging_item)
                                    self.dragging_item = None
                            else:
                                self.dragging_item["slot"] = self.hover_rect[1]
                                self.dragging_item = current_slot
                                current_slot["slot"] = 0 # 0 means it is not in the inventory     
                        else:
                            self.items.append({
                                "slot": self.hover_rect[1],
                                "item": self.dragging_item["item"],
                                "img": self.dragging_item["img"],
                                "amount": 1
                            })                       
                            self.dragging_item["amount"] -= 1
                            if self.dragging_item["amount"] == 0:
                                self.items.remove(self.dragging_item)
                                self.dragging_item = None          
                        self.selected_slot = self.hover_rect[1]       
                    elif is_origin_slot:
                        self.dragging_item = None  
        except TypeError:
            pass

    def dragging_item_drop(self, btn):
        if btn == 1:
            if self.dragging_item != None and self.hover_rect != None:
                if self.check_slot(self.hover_rect[1]):
                    self.check_slot(self.hover_rect[1])["slot"] = self.dragging_item["slot"] 
                self.dragging_item["slot"] = self.hover_rect[1]
                self.selected_slot = self.hover_rect[1]
            self.dragging_item = None

    def check_slot(self, slot):
        for i in self.items:
            if i["slot"] == slot:
                return i
        return False

    def get_item(self, item_id):
        for i in self.items:
            if i["item"].id == item_id:
                return i
        return False

    def draw_item(self, win, slot, rect):
        item = self.check_slot(slot)
        if item:
            win.blit(item["img"], (rect.center[0]-item["item"].width/2, rect.center[1]-item["item"].height/2))
            if len(str(item["amount"])) == 1:
                offset = (16, 24)
            elif len(str(item["amount"])) == 2:
                offset = (20, 24)
            elif len(str(item["amount"])) == 3:
                offset = (28, 24)
            draw_text(win, (rect.right-offset[0], rect.bottom-offset[1]), str(item["amount"]), 16, BLACK, "assets/opensans.ttf")
        else:
            pass

    def hover_item(self, rect, slot, win):
        item = self.check_slot(slot)
        if mouse_over(pygame.mouse.get_pos(), rect.left, rect.top, rect.width, rect.height):
            if item and self.dragging_item == None:
                text = item["item"].name
                color = (255, 255, 255)
                font = pygame.font.Font('assets/font.ttf', 32-len(text))
                info_box = pygame.Rect(rect.left+30, rect.top-30, 140, 32)
                txt_surface = font.render(text, True, color)
                win.blit(txt_surface, (info_box.x+5, info_box.y+5))
                pygame.draw.rect(win, color, info_box, 3) 

    def add_item(self, item, amount=1):
        item_exists = self.get_item(item.id)
        if item_exists: 
            item_exists["amount"] += amount
        else:
            item_info = {
                "slot": self.next_slot,
                "item": item,
                "img": item.image,
                "amount": amount
            }
            self.items.append(item_info)
            self.next_slot += 1