import pygame
import json
from python.particles import Particle
import random


# melee attack doesn't work, realized last minute that it's broken so I decided to make all attacks ranged.
# if you want to fix it, go ahead all the code is there but the animation is broken and so is the enemy following the player until they are next to each other


class Enemies():
    def __init__(self):
        self.load_enemies()
        self.enemies_spawned = []

    def update(self, dt, player):
        for enemy in self.enemies_spawned:
            enemy.update(dt, player)

    def render(self, win, scroll, dt, time, show_outline):
        for enemy in self.enemies_spawned:
            enemy.render(win, scroll, dt, time, show_outline)

    def load_enemies(self):
        with open("assets/json/enemies.json") as f:
            self.enemies = json.load(f)
        self.art = {}
        for enemy in self.enemies:
            art_folder = self.enemies[enemy]["art folder"]
            idle_anim_name = self.enemies[enemy]["idle anim"]
            walk_anim_name = self.enemies[enemy]["walk anim"]
            self.art[enemy] = {}
            self.art[enemy]["animations"] = [[], [], [], []] # idle right, idle left, walk right, walk left
            for x in range(self.enemies[enemy]["frames"]):
                idle_right = pygame.transform.scale(pygame.image.load(f"{art_folder}/{idle_anim_name}{x+1}.png").convert_alpha(), self.enemies[enemy]["dimensions"])
                walk_right = pygame.transform.scale(pygame.image.load(f"{art_folder}/{walk_anim_name}{x+1}.png").convert_alpha(), self.enemies[enemy]["dimensions"])
                idle_left = pygame.transform.flip(idle_right, True, False)
                walk_left = pygame.transform.flip(walk_right, True, False)
                self.art[enemy]["animations"][0].append(idle_right)
                self.art[enemy]["animations"][1].append(idle_left)
                self.art[enemy]["animations"][2].append(walk_right)
                self.art[enemy]["animations"][3].append(walk_left)


class Enemy():
    def __init__(self, x, y, id, enemy, animations, particles):
        self.particles = particles
        self.id = id
        self.enemy = enemy
        self.name = self.enemy["name"]
        self.width = self.enemy["dimensions"][0]
        self.height = self.enemy["dimensions"][1]
        self.health = self.enemy["health"]
        self.max_health = self.enemy["health"]
        self.speed = self.enemy["speed"]
        self.damage = self.enemy["damage"]
        self.los = self.enemy["los"] # line of sight
        self.x = x
        self.y = y
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        animations = animations["animations"]
        self.anim_idle_right = animations[0]
        self.anim_idle_left = animations[1]
        self.anim_walk_right = animations[2]
        self.anim_walk_left = animations[3]
        self.frames = self.enemy["frames"]
        self.frame = 0
        self.last_direction = "right"
        self.last_y_direction = None
        self.moving = False
        self.move_x = True
        self.draw_health = False
        self.attacking = False
        self.attack_animation = [False, 0]
        self.attack_cooldown = 0
        self.attack_reset = False # has attacked the player once already in the current animation
        self.weapon_img = pygame.transform.scale(pygame.image.load("assets/items/iron_sword_lunge.png").convert_alpha(), (56, 20))
        self.weapon_rect = pygame.Rect(0, 0, 0, 0)
        self.projectiles = []
        self.since_last_change = 0

    def update(self, dt, player):
        self.AI(player, dt)
        for projectile in self.projectiles:
            x = projectile.update(dt, self.particles)
            if self.enemy["weapon"] == "potion" or self.enemy["weapon"] == "arrow":
                collided = player.rect.colliderect(projectile.rect)
            elif self.enemy["weapon"] == "pistol":
                collided = player.rect.collidepoint(projectile.x, projectile.y)
            if collided:
                player.health -= self.damage
                if player.health <= 0:
                    player.die()
                try:
                    projectile.explode(self.particles)
                except:
                    pass
                self.projectiles.remove(projectile)
            else:
                if not x:
                    self.projectiles.remove(projectile)

        if self.attacking and self.enemy["attack type"] == "melee":
            if not round((self.attack_cooldown * dt)) <= 0:
                self.attack_cooldown -= 1
            else:
                self.attack_animation[0] = True
            if self.attack_reset == False:
                self.attack_collisions(player)
            self.moving = False
        elif self.attacking and self.enemy["attack type"] == "ranged":
            if round((self.attack_cooldown * dt)) >= 120:
                self.throw_projectile(player)
                self.attack_cooldown = 0
            else:
                self.attack_cooldown += 1
            self.last_direction = "right" if player.x > self.x else "left"
            self.moving = False
        else:
            if self.enemy["attack type"] == "ranged" and self.isInLine((player.x, player.y), self.enemy["attack range"]-100):
                if round((self.attack_cooldown * dt)) >= 120:
                    if self.last_direction == "right":
                        rand_x = random.randint(player.x+100, player.x+300)
                    else:
                        rand_x = random.randint(player.x-300, player.x-100)
                    self.throw_projectile(player, (rand_x, random.randint(player.y-100, player.y+100)))
                    self.attack_cooldown = 0
                else:
                    self.attack_cooldown += 1                
            self.move()

    def throw_projectile(self, player, coords=None):
        if coords != None:
            x, y = coords
        else:
            x, y = player.x, player.y
        self.last_direction = "right" if x > self.x else "left"
        if self.enemy["weapon"] == "potion":
            projectile = Projectile(pygame.Vector2(self.x, self.y), pygame.Vector2(x, y), "assets/items/potion_damage.png", (32,32))
        elif self.enemy["weapon"] == "arrow":
            if -50 < player.x - self.x < 50:
                last_dir = None
            else:
                last_dir = self.last_direction
            if -50 < player.y - self.y < 50:
                up = None
            elif player.y > self.y:
                up = False
            else:
                up = True
            projectile = Arrow(self.x, self.y+self.rect.height/2, last_dir, up)
        elif self.enemy["weapon"] == "pistol":
            if -50 < player.x - self.x < 50:
                last_dir = None
            else:
                last_dir = self.last_direction
            if -50 < player.y - self.y < 50:
                up = None
            elif player.y > self.y:
                up = False
            else:
                up = True
            projectile = Bullet(self.x+self.rect.width/2, self.y+self.height/2, last_dir, up)
        self.projectiles.append(projectile)

    def attack_collisions(self, player):
        if self.weapon_rect.colliderect(player.rect) and self.attack_reset == False and self.attack_animation[0] == True:
            player.health -= self.damage
            self.attack_reset = True

    def die(self, enemies, player):
        player.balance += random.randint(50, 100)
        enemies.enemies_spawned.remove(self)
        for x in range(random.randint(10,25)):   
            particle = Particle((self.x+self.rect.width/2+random.randint(-5, 5), self.y+self.rect.height/2+random.randint(-5, 5)), 
            [random.randint(15, 30) / 10 - 1, random.randint(-4, -1)], random.randint(2,6), (219,181,30)) 
            self.particles.particles.append(particle)

    def move(self):
        if self.moving:
            if self.move_x:
                if self.last_direction == "right":
                    if self.x > 100*32-32:
                        return
                    self.x += self.speed
                    self.rect.x += self.speed
                elif self.last_direction == "left":
                    if self.x < 0:
                        return
                    self.x -= self.speed
                    self.rect.x -= self.speed
            if self.last_y_direction != None:
                if self.last_y_direction == "up":
                    if self.y < 0:
                        return
                    self.y -= self.speed
                    self.rect.y -= self.speed
                elif self.last_y_direction == "down":
                    if self.y > 100*32-32:
                        return
                    self.y += self.speed
                    self.rect.y += self.speed

    def AI(self, player, dt):
        if self.enemy["attack type"] == "melee":
            range = self.rect.w/2
        else:
            range = self.enemy["attack range"]
        is_in_attack_range = self.isInLine([player.x, player.y], range)
        is_in_sight = self.isInLine([player.x, player.y], self.los)
        if is_in_attack_range:
            self.draw_health = True
            self.moving = False
            self.attacking = True
            self.last_direction = "right" if player.x > self.x else "left"
        elif is_in_sight:
            self.draw_health = True
            if -20 < player.x - self.x < 20:
                self.last_direction = "right"
                self.move_x = False
            elif player.x > self.x: # Entity is to the right
                self.last_direction = "right"
                self.move_x = True
            else:
                self.last_direction = "left"
                self.move_x = True
            if -20 < player.y - self.y < 20:
                self.last_y_direction = None
            elif player.y > self.y:
                self.last_y_direction = "down"
            elif player.y < self.y:
                self.last_y_direction = "up"

            self.moving = True
            self.attacking = False
            self.draw_health = True
        else:
            if self.enemy["explorer"]:
                self.moving = True
                self.attacking = False
                self.draw_health = False
                if round((self.since_last_change * dt)) >= random.randint(120,360):
                    self.last_direction = random.choice(["left", "right"])
                    self.last_y_direction = random.choice(["up", "down", None])
                    self.since_last_change = 0
                else:
                    self.since_last_change += 1
                if self.x > 100*32-32:
                    self.last_direction = "left"
                elif self.x < 0:
                    self.last_direction = "right"
                if self.y > 100*32-32:
                    self.last_y_direction = "up"
                elif self.y < 0:
                    self.last_y_direction = "down"
            else:
                self.draw_health = False
                self.moving = False
                self.attacking = False

    def isInLine(self, entity, range): # Checks if entity is in given range
        entityDX = abs(entity[0] - self.x)
        entityDY = abs(entity[1] - self.y)
        if entityDX <= range and entityDY <= range:
            return True
        return False

    def render(self, win, scroll, dt, time, show_outline):
        for projectile in self.projectiles:
            projectile.render(win, scroll, show_outline)

        if time % 2:
            if self.frame + 1 >= self.frames*3:
                self.frame = 0
            else:
                self.frame += 1
        
        if show_outline:
            pygame.draw.rect(win, (255, 0, 0), self.rect, 1, border_radius=1)

        coords = self.rect.topleft - scroll
        if self.moving:
            if self.last_direction == "right":
                win.blit(self.anim_walk_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_walk_left[self.frame//3], coords)
        else:
            if self.last_direction == "right":
                win.blit(self.anim_idle_right[self.frame//3], coords)
            elif self.last_direction == "left":
                win.blit(self.anim_idle_left[self.frame//3], coords)
        
        if self.enemy["attack type"] == "melee":
            if self.attack_animation[0]:
                if self.last_direction == "right":
                    win.blit(self.weapon_img, (self.x-scroll.x+self.rect.w/2+5, self.y-scroll.y+self.rect.h/2+15))
                    self.weapon_rect = pygame.Rect(self.x+self.rect.w/2+5, self.y+self.rect.h/2+15, 56, 20)
                else:
                    win.blit(pygame.transform.flip(self.weapon_img, True, False), (self.x-scroll.x-56/4, self.y-scroll.y+self.rect.h/2+15))
                    self.weapon_rect = pygame.Rect(self.x-56/4, self.y+self.rect.h/2+15, 56, 20)
                if round((self.attack_animation[1] * dt)) >= 50: # animation is over
                    self.attack_animation[0] = False
                    self.attack_animation[1] = 0
                    self.attack_cooldown = self.enemy["attack cooldown"] # in seconds
                    self.attack_reset = False
                    self.attacking = False
                else:
                    self.attack_animation[1] += 1

        if self.draw_health:
            health_rect = pygame.Rect(0, 0, 42, 7)
            health_rect.midbottom = self.rect.centerx-scroll.x, self.rect.top-scroll.y
            self.draw_health_bar(win, health_rect.topleft, health_rect.size, 
                (0, 0, 0), (255, 0, 0), (0, 255, 0), self.health/self.max_health)  

    def draw_health_bar(self, surf, pos, size, borderC, backC, healthC, progress):
        pygame.draw.rect(surf, backC, (*pos, *size))
        pygame.draw.rect(surf, borderC, (*pos, *size), 1)
        innerPos  = (pos[0]+1, pos[1]+1)
        innerSize = ((size[0]-2) * progress, size[1]-2)
        rect = (round(innerPos[0]), round(innerPos[1]), round(innerSize[0]), round(innerSize[1]))
        pygame.draw.rect(surf, healthC, rect)


class Bullet():
    def __init__(self, x, y, direction, up):
        self.x = x
        self.y = y 
        self.direction = direction
        self.speed = 12
        self.color = (255, 255, 255)
        self.time = 0
        self.up = up

    def render(self, win, scroll, debug=None):
        pygame.draw.circle(win, self.color, (self.x - scroll.x, self.y - scroll.y), 2)

    def update(self, dt, particles):
        if self.direction == "right":
            self.x += self.speed
        elif self.direction == "left":
            self.x -= self.speed
        if self.up:
            self.y -= self.speed
        elif self.up == False:
            self.y += self.speed
        else:
            pass
        self.time += 1
        if round(self.time * dt) >= 120:
            return False
        return True

    def explode(self, particles):
        for x in range(random.randint(10,25)):   
            particle = Particle((self.target_pos[0]+self.rect.width/2+random.randint(-5, 5), self.target_pos[1]+self.rect.height/2+random.randint(-5, 5)), 
            [random.randint(15, 30) / 10 - 1, random.randint(-4, -1)], random.randint(2,6), (0,0,0)) 
            particles.particles.append(particle)

class Arrow():
    def __init__(self, x, y, direction, up):
        self.x = x
        self.y = y 
        self.direction = direction
        self.up = up
        self.speed = 12
        self.color = (255, 255, 255)
        self.time = 0
        self.img = pygame.transform.scale(pygame.image.load("assets/items/arrow.png"), (16+16, 7+16))
        if direction == "left":
            self.img = pygame.transform.flip(self.img, True, False)
        if up == True:
            self.img = pygame.transform.rotate(self.img, 90)
        elif up == False:
            self.img = pygame.transform.rotate(self.img, -90)
        self.rect = pygame.Rect(self.x, self.y, 16+16, 7+16)

    def render(self, win, scroll, debug=None):
        win.blit(self.img, (self.x - scroll.x, self.y - scroll.y))

    def update(self, dt, particles):
        if self.direction == "right":
            self.x += self.speed
        elif self.direction == "left":
            self.x -= self.speed
        if self.up:
            self.y -= self.speed
        elif self.up == False:
            self.y += self.speed
        else:
            pass
        self.rect.x = self.x
        self.rect.y = self.y
        self.time += 1
        if round(self.time * dt) >= 120:
            return False
        return True

class Projectile(): # Math for arrow made by Matiiss#5629 from the pygame discord server
    def __init__(self, initial_pos, target_pos, image_path, image_dimensions):
        self.target_pos = target_pos
        self.img = pygame.transform.scale(pygame.image.load(image_path).convert_alpha(), image_dimensions)
        self.rect = self.img.get_rect(center=initial_pos)

        self.y_zero = initial_pos[1]
        self.point = target_pos - initial_pos
        self.acceleration = 1
        point_x = self.point[0]
        arch_height = 8 # smaller = larger arch
        self.x_vel = point_x and arch_height * (abs(point_x) / point_x)

        self.y_vel, self.frames = self.calculate_y_vel_and_frame_count()
        self.frame_count = 0

    def update(self, dt, particles):
        if self.frame_count > self.frames:
            self.explode(particles)
            return False
        self.frame_count += 1

        t = self.frame_count
        a = self.acceleration
        y0 = self.y_zero
        y = self.y_vel * t + 0.5 * a * t ** 2 + y0

        self.rect.move_ip(self.x_vel, 0)
        self.rect.y = y
        return True
    
    def render(self, win, scroll, debug):
        angle = self.frames-self.frame_count*2
        win.blit(pygame.transform.rotate(self.img, angle), self.rect.topleft-scroll)
        if debug: pygame.draw.rect(win, (255, 0, 0), self.rect, 1)

    def calculate_y_vel_and_frame_count(self):
        x, y = self.point
        x_vel = self.x_vel
        a = self.acceleration

        t = max(1, x_vel and x / x_vel)
        b = (y - (0.5 * a * t ** 2)) / t
        return b, abs(t)

    def explode(self, particles):
        for x in range(random.randint(10,25)):   
            particle = Particle((self.target_pos[0]+self.rect.width/2+random.randint(-5, 5), self.target_pos[1]+self.rect.height/2+random.randint(-5, 5)), 
            [random.randint(15, 30) / 10 - 1, random.randint(-4, -1)], random.randint(2,6), (162,30,219)) 
            particles.particles.append(particle)