import pygame
from python.utils import mouse_over, draw_large_text, draw_text


INV_EMPTY = (23, 88, 101)
INV_NOT_EMPTY = (46, 209, 239)
INV_SELECTED = (255,215,0)
INV_HOVER = (0, 215, 255)
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)

class HUD():
    def __init__(self, screen_dimensions):
        self.width = screen_dimensions[0]
        self.height = screen_dimensions[1]
        self.inventory_open = False
        self.inv = Inventory(screen_dimensions)
        self.coin_img = pygame.transform.scale(pygame.image.load("assets/items/coin.png").convert_alpha(), (50, 50))
        self.bullet_img = pygame.transform.scale(pygame.image.load("assets/items/bullet.png").convert_alpha(), (50,50))

    def render(self, win, player, enemies, tutorial=False):

        # player health bar
        if tutorial:
            draw_text(win, (self.width/2-100, self.height - 50-40), f"Your Health", 30, (0,255,0), "assets/font.ttf")
        progress = player.health / player.max_health
        s = pygame.Surface((200, 20))
        s.set_alpha(200)
        s.fill((165, 49, 49))
        #pygame.draw.rect(s, (165, 49, 49), (self.width / 2 - 100, self.height - 50, 200, 20))
        win.blit(s, (self.width / 2 - 100, self.height - 50))
        pygame.draw.rect(win, (216, 36, 36), (self.width / 2 - 100, self.height - 50, 200*progress, 20))
        pygame.draw.rect(win, (0, 0, 0), (self.width / 2 - 100, self.height - 50, 200, 20), 5)
        draw_text(win, (self.width / 2 + 120, self.height - 50), f"{player.health}/{player.max_health}", 30, (0,0,0), "assets/font.ttf")

        # player ammo
        if tutorial:
            draw_text(win, (125, self.height - 65-40), f"Your Ammo", 30, (0,255,0), "assets/font.ttf")
        draw_text(win, (125, self.height - 65), f"{player.loaded_ammo}/{player.ammo}", 50, WHITE, "assets/font.ttf")
        if len(str(player.ammo)) in [1, 2, 3]:
            offset = 60
        elif len(str(player.balance)) == 4:
            offset = 70
        elif len(str(player.balance)) == 5:
            offset = 80
        elif len(str(player.balance)) == 6:
            offset = 90
        win.blit(self.bullet_img, (125-offset, self.height - 65))        

        # player balance
        if tutorial:
            draw_text(win, (self.width-200, self.height - 65-40), f"Your Money", 30, (0,255,0), "assets/font.ttf")
        draw_text(win, (self.width - 125, self.height - 65), f"{player.balance}", 50, WHITE, "assets/font.ttf")
        if len(str(player.balance)) in [1, 2, 3]:
            offset = 180
        elif len(str(player.balance)) == 4:
            offset = 190
        elif len(str(player.balance)) == 5:
            offset = 200
        elif len(str(player.balance)) == 6:
            offset = 210
        win.blit(self.coin_img, (self.width - offset, self.height - 65))

        # enemies left
        if not tutorial:
            draw_text(win, (50, 50), f"{len(enemies.enemies_spawned)} enemies left", 35, WHITE, "assets/font.ttf")

        # player inventory
        if self.inventory_open:
            self.inv.render_inv(win)

class Inventory():
    def __init__(self, screen_dimensions):
        self.win_size = screen_dimensions
        self.menu = "inventory"
        self.menu_btns = []
        self.inventory = []
        self.inv_box_size = (50,50)
        self.next_slot = 1
        self.selected_slot = 0
        self.dragging_item = None
        self.hover_rect = None
        self.rendered_items = []
        self.use_btn = None

    def render_inv(self, win):
        y = 0
        x = 1
        init_x = (self.win_size[0]/2-(self.inv_box_size[0]*10/2))-300
        init_y = self.win_size[1]/3
        self.hover_rect = None
        for z in range(70):
            color = INV_NOT_EMPTY if self.check_slot(z+1) else INV_EMPTY
            rect = pygame.draw.rect(win, color,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]))

            outline_rect = pygame.draw.rect(win, INV_SELECTED if self.selected_slot == z+1 else BLACK,
            (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
            self.inv_box_size[0], self.inv_box_size[1]), 4)

            self.draw_item(win, z+1, rect)
            if mouse_over(pygame.mouse.get_pos(), outline_rect.left, outline_rect.top, outline_rect.width, outline_rect.height):
                self.hover_rect = (rect, z+1) # This is needed so that it draws the hover box on top of the inventory
                if self.dragging_item != None: # dragging item rect outline
                    outline_rect = pygame.draw.rect(win, INV_HOVER,
                    (init_x+x*(self.inv_box_size[0]+4), init_y+y, 
                    self.inv_box_size[0], self.inv_box_size[1]), 4)

            if str(x).endswith("0") and x != 0:
                y += self.inv_box_size[1]+4.1
                x = 1
            else:
                x += 1
        
        self.preview_box(win)
        #self.sort_buttons(win)
        self.render_use_btn(win)
        if self.hover_rect != None:
            self.hover_item(self.hover_rect[0], self.hover_rect[1], win)
        
        if self.dragging_item != None:
            img = self.dragging_item["img"].copy()
            img.fill((255, 255, 255, 200), None, pygame.BLEND_RGBA_MULT)
            win.blit(img, (pygame.mouse.get_pos()[0]-img.get_width()/2, pygame.mouse.get_pos()[1]-img.get_height()/2))

    def render_use_btn(self, win):
        x = (self.win_size[0]/2-(self.inv_box_size[0]*10/2))-245
        y = self.win_size[1]/3-59

        font = pygame.font.Font('assets/font.ttf', 32-len("Use"))
        info_box = pygame.Rect(x, y, 55, 32)
        txt_surface = font.render("Use", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED, info_box, 3) 
        self.use_btn = rect

    def preview_box(self, win):
        item = self.check_slot(self.selected_slot)
        rect = pygame.draw.rect(win, BLACK,
        (int(self.win_size[0]-500), int(self.win_size[1]/3), 
        350, 400))

        if item != False:
            w = 200
            h = 200
            win.blit(pygame.transform.scale(item["img"], (w, h)), (rect.center[0]-w/2+15, rect.center[1]-h/1.5))    
            draw_text(win, (rect.center[0]-w/2+15, rect.center[1]-h/1.5-24), item["item"].name, 24, WHITE, "assets/font.ttf")
            #draw_text(win, (rect.left+20, rect.center[1]+40), item["item"].item["description"], 20, WHITE, "assets/font.ttf")
            font = pygame.font.Font("assets/font.ttf", 20)
            draw_large_text(win, item["item"].item["description"], (rect.left+20, rect.center[1]+40), font, WHITE)
            draw_text(win, (rect.left+20, rect.center[1]+80), "Type: " + item["item"].item["type"], 20, WHITE, "assets/font.ttf")

            item_dict = item["item"].item
            x, y = rect.left+20, rect.center[1]+100
            for effect in item_dict["effects"]:
                draw_text(win, (x, y), f"{effect.capitalize()}: " + str(item_dict[effect]), 20, WHITE, "assets/font.ttf")
                y += 20

    def sort_buttons(self, win):
        x = (self.win_size[0]/2-(self.inv_box_size[0]*10/2))-245
        y = self.win_size[1]/3-59

        # inventory button
        font = pygame.font.Font('assets/font.ttf', 32-len("Inventory"))
        info_box = pygame.Rect(x, y, len("inventory")*16-len("inventory")*4, 32)
        txt_surface = font.render("Inventory", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED if self.menu == "inventory" else WHITE, info_box, 3) 
        self.menu_btns.append([rect, "inventory"])

        # weapons button
        font = pygame.font.Font('assets/font.ttf', 32-len("Weapons"))
        info_box = pygame.Rect(rect.right+30, y, len("weapons")*16-len("weapons")*2, 32)
        txt_surface = font.render("Weapons", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED if self.menu == "weapons" else WHITE, info_box, 3) 
        self.menu_btns.append([rect, "weapons"])

        # food button
        font = pygame.font.Font('assets/font.ttf', 32-len("Food"))
        info_box = pygame.Rect(rect.right+30, y, len("food")*18-len("food")*2, 32)
        txt_surface = font.render("Food", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED if self.menu == "food" else WHITE, info_box, 3) 
        self.menu_btns.append([rect, "food"])
        
        # potions button
        font = pygame.font.Font('assets/font.ttf', 32-len("potions"))
        info_box = pygame.Rect(rect.right+30, y, len("potions")*16-len("potions")*3, 32)
        txt_surface = font.render("Potions", True, WHITE)
        win.blit(txt_surface, (info_box.x+5, info_box.y+5))
        rect = pygame.draw.rect(win, INV_SELECTED if self.menu == "potions" else WHITE, info_box, 3) 
        self.menu_btns.append([rect, "potions"])

    def on_click(self, btn, player):
        try:
            if btn == 1: # Left click
                if self.dragging_item == None:
                    if self.use_btn != None and self.check_slot(self.selected_slot) != False:
                        if mouse_over(pygame.mouse.get_pos(), self.use_btn.left, self.use_btn.top, self.use_btn.width, self.use_btn.height):
                            player.used_item = True
                            item_dict = self.check_slot(self.selected_slot)["item"].item
                            if item_dict["effects"] != []:
                                for effect in item_dict["effects"]:
                                    if effect == "heal":
                                        player.health += item_dict[effect]
                                        if player.health > player.max_health:
                                            player.health = player.max_health
                                    elif effect == "strength boost":
                                        player.damage += item_dict[effect]
                                    elif effect == "speed boost":
                                        player.speed += item_dict[effect]
                            if item_dict["type"] == "ammo":
                                player.ammo += item_dict["contains"]
                            elif item_dict["type"] == "coin":
                                player.balance += item_dict["contains"]
                            self.check_slot(self.selected_slot)["amount"] -= 1
                            if self.check_slot(self.selected_slot)["amount"] <= 0:
                                self.inventory.remove(self.check_slot(self.selected_slot))

                    for b in self.menu_btns:
                        rect = b[0]
                        if mouse_over(pygame.mouse.get_pos(), rect.left, rect.top, rect.width+4, rect.height+4):
                            self.rendered_items = []
                            if b[1] != "inventory":
                                if b[1] == "weapons": tag = "weapon"
                                elif b[1] == "food": tag = "food"
                                elif b[1] == "potions": tag = "potion"
                                for i in self.inventory:
                                    if i["item"].item["type"] == tag:
                                        self.rendered_items.append(i)
                            self.menu = b[1]
                            self.selected_slot = None
                            break
                # Dragging item
                if self.menu == "inventory" and self.dragging_item == None and self.selected_slot != 0 and self.selected_slot != None and self.check_slot(self.selected_slot) != False:
                    if mouse_over(pygame.mouse.get_pos(), self.hover_rect[0].left, self.hover_rect[0].top, self.hover_rect[0].width+4, self.hover_rect[0].height+4):
                        selected_slot = self.hover_rect[1]
                    else:
                        selected_slot = None
                    if self.selected_slot == selected_slot:
                        s = self.check_slot(self.selected_slot)
                        self.dragging_item = s if s is not False else None
                # Dropping item
                elif self.menu == "inventory" and self.dragging_item != None and self.hover_rect != None:
                    is_origin_slot = bool(self.hover_rect[1] == self.dragging_item["slot"])
                    if not is_origin_slot:
                        current_slot = self.check_slot(self.hover_rect[1])
                        if current_slot != False:
                            if current_slot["item"].id == self.dragging_item["item"].id:
                                current_slot["amount"] += self.dragging_item["amount"]
                                self.inventory.remove(self.dragging_item)
                                self.dragging_item = None
                            else:
                                self.dragging_item["slot"] = self.hover_rect[1]
                                self.dragging_item = current_slot
                                current_slot["slot"] = 0 # 0 means it is not in the inventory
                        else:
                            self.dragging_item["slot"] = self.hover_rect[1]
                            self.dragging_item = None
                        self.selected_slot = self.hover_rect[1]
                    elif is_origin_slot:
                        self.dragging_item = None
                # Selecting a slot
                if self.hover_rect != None:
                    if mouse_over(pygame.mouse.get_pos(), self.hover_rect[0].left, self.hover_rect[0].top, self.hover_rect[0].width+4, self.hover_rect[0].height+4):
                        self.selected_slot = self.hover_rect[1]

            elif btn == 2: # Middle click
                pass
                                
            elif btn == 3: # Right click
                if self.menu == "inventory" and self.dragging_item != None and self.hover_rect != None: # dropping only 1 item from stack
                    is_origin_slot = bool(self.hover_rect[1] == self.dragging_item["slot"])
                    if not is_origin_slot:
                        current_slot = self.check_slot(self.hover_rect[1])
                        if current_slot != False:
                            if current_slot["item"].id == self.dragging_item["item"].id:
                                current_slot["amount"] += 1
                                self.dragging_item["amount"] -= 1
                                if self.dragging_item["amount"] == 0:
                                    self.inventory.remove(self.dragging_item)
                                    self.dragging_item = None
                            else:
                                self.dragging_item["slot"] = self.hover_rect[1]
                                self.dragging_item = current_slot
                                current_slot["slot"] = 0 # 0 means it is not in the inventory     
                        else:
                            self.inventory.append({
                                "slot": self.hover_rect[1],
                                "item": self.dragging_item["item"],
                                "img": self.dragging_item["img"],
                                "amount": 1
                            })                       
                            self.dragging_item["amount"] -= 1
                            if self.dragging_item["amount"] == 0:
                                self.inventory.remove(self.dragging_item)
                                self.dragging_item = None          
                        self.selected_slot = self.hover_rect[1]       
                    elif is_origin_slot:
                        self.dragging_item = None  
        except TypeError:
            pass

    def dragging_item_drop(self, btn):
        if btn == 1:
            if self.dragging_item != None and self.hover_rect != None:
                if self.check_slot(self.hover_rect[1]):
                    self.check_slot(self.hover_rect[1])["slot"] = self.dragging_item["slot"] 
                self.dragging_item["slot"] = self.hover_rect[1]
                self.selected_slot = self.hover_rect[1]
            self.dragging_item = None

    def check_slot(self, slot):
        if self.menu == "inventory":
            for i in self.inventory:
                if i["slot"] == slot:
                    return i
            return False
        else:
            try:
                return self.rendered_items[slot-1]
            except:
                return False

    def get_item(self, item_id):
        for i in self.inventory:
            if i["item"].id == item_id:
                return i
        return False

    def draw_item(self, win, slot, rect):
        item = self.check_slot(slot)
        if item:
            win.blit(item["img"], (rect.center[0]-item["item"].width/2, rect.center[1]-item["item"].height/2))
            if len(str(item["amount"])) == 1:
                offset = (16, 24)
            elif len(str(item["amount"])) == 2:
                offset = (20, 24)
            elif len(str(item["amount"])) == 3:
                offset = (28, 24)
            draw_text(win, (rect.right-offset[0], rect.bottom-offset[1]), str(item["amount"]), 16, BLACK, "assets/opensans.ttf")
        else:
            pass

    def hover_item(self, rect, slot, win):
        item = self.check_slot(slot)
        if mouse_over(pygame.mouse.get_pos(), rect.left, rect.top, rect.width, rect.height):
            if item and self.dragging_item == None:
                text = item["item"].name
                color = (255, 255, 255)
                font = pygame.font.Font('assets/font.ttf', 32-len(text))
                info_box = pygame.Rect(rect.left+30, rect.top-30, 140, 32)
                txt_surface = font.render(text, True, color)
                win.blit(txt_surface, (info_box.x+5, info_box.y+5))
                pygame.draw.rect(win, color, info_box, 3) 

    def add_item(self, item):
        item_exists = self.get_item(item.id)
        if item_exists: item_exists["amount"] += 1
        else:
            item_info = {
                "slot": self.next_slot,
                "item": item,
                "img": item.image,
                "amount": 1
            }
            self.inventory.append(item_info)
            self.next_slot += 1