import pygame
import sys
from python.utils import draw_text, mouse_over
from python.player import Player
from python.hud import HUD
from python.enemies import Enemies, Enemy
from python.items import Items, Item
from python.chests import Chest
from python.particles import Particles


class menu:
    def __init__(self):
        self.start = Button((0,255,0), 1280/2, 720/2-200, 145, 75, "Start", 40)
        self.tutorial = Button((0,0,255), 1280/2, 720/2, 145, 75, "Tutorial", 40)
        self.in_tutorial = False
        self.items = Items()
        self.chest = Chest((1280,720), (1280/2+150, 720/2))
        self.particles = Particles()
        self.chest.add_item(Item("coin pack", self.items.items["coin pack"], self.items.art["coin pack"]))
        self.chest.add_item(Item("ammo pack", self.items.items["ammo pack"], self.items.art["ammo pack"]))
        self.enemies = Enemies()
        self.player = Player((1280,720), (1280/2, 720/2), None)
        self.hud = HUD((1280, 720))
        self.hud.inv.add_item(Item("healing_potion", self.items.items["healing_potion"], self.items.art["healing_potion"]))
        self.time = 0
        self.dt = 0
        self.clock = pygame.time.Clock()

    def render(self, win):
        if not self.in_tutorial:
            win.fill((0,0,0))

            self.start.draw(win, (0,255,0))
            self.tutorial.draw(win, (0,0,255))

            pygame.display.update()
        else:
            win.fill((0,0,0))

            self.player.render(win, pygame.Vector2(0,0), self.dt, self.time, False)
            self.enemies.render(win, pygame.Vector2(0,0), self.dt, self.time, False)
            if self.player.moved:
                self.hud.render(win, self.player, self.enemies, True)
            if self.player.used_item:
                self.chest.render(win, pygame.Vector2(0,0))
                if self.chest.open:
                    self.chest.render_inv(win)
            
            if not self.player.moved:
                draw_text(win, (100, 100), "Use WASD or Arrow Keys to move", 25, (255,255,255), "assets/opensans.ttf")
            elif not self.player.opened_inv:
                draw_text(win, (100, 100), "Click E to open your inventory and press E or ESC to close it", 25, (255,255,255), "assets/opensans.ttf")               
                draw_text(win, (100, 150), "You can move items around by double clicking them", 25, (255,255,255), "assets/opensans.ttf")                
            elif not self.player.used_item:
                draw_text(win, (100, 100), "Select the health potion and click USE to fill up your health", 25, (255,255,255), "assets/opensans.ttf")   
                draw_text(win, (100, 150), "You can move items around by double clicking them", 25, (255,255,255), "assets/opensans.ttf")              
            elif not self.player.opened_chest:
                draw_text(win, (100, 100), "Click on the chest to open it and take the items", 25, (255,255,255), "assets/opensans.ttf")   
                draw_text(win, (100, 150), "You can use money to buy items from the merchant", 25, (255,255,255), "assets/opensans.ttf")      
            elif not self.player.shot:
                draw_text(win, (100, 100), "Press ESCAPE to close the chest", 25, (255,255,255), "assets/opensans.ttf")   
                draw_text(win, (100, 150), "Left Click to bring up your pistol, Left Click again to shoot", 25, (255,255,255), "assets/opensans.ttf")   
            elif not self.player.attacked:
                draw_text(win, (100, 100), "Kill the enemy in front of you by shooting them", 25, (255,255,255), "assets/opensans.ttf")   
            elif self.player.attacked:
                draw_text(win, (100, 100), "Congrats! The battlefield is waiting!", 25, (255,255,255), "assets/opensans.ttf")   
                draw_text(win, (100, 150), "Press ENTER to Start", 25, (255,255,255), "assets/opensans.ttf")   

            pygame.display.update()

    def update(self):
        self.dt = self.clock.tick(60) * 0.001 * 60
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_e:
                    if not self.chest.open:
                        self.player.opened_inv = True
                        self.hud.inventory_open = not self.hud.inventory_open
                if event.key == pygame.K_ESCAPE:
                    self.hud.inventory_open = False
                    self.chest.open = False
                if event.key == pygame.K_RETURN and self.player.attacked:
                    return "start"
            if event.type == pygame.MOUSEBUTTONDOWN:
                if not self.in_tutorial:
                    if self.start.isOver(pygame.mouse.get_pos()):
                        return "start"
                    elif self.tutorial.isOver(pygame.mouse.get_pos()):
                        return "tutorial"
                else:
                    if not self.hud.inventory_open and not self.chest.open and self.player.opened_chest == True:
                        if event.button == 1:
                            if self.player.gun_equiped:
                                self.player.attack()
                                self.player.since_gun_equiped = 0
                                if self.player.shot == False:
                                    self.enemies.enemies_spawned.append(Enemy(1280/2+400, 720/2, "large mushroom", self.enemies.enemies["large mushroom"], self.enemies.art["large mushroom"], self.particles))
                                self.player.shot = True
                            else:
                                self.player.gun_equiped = True
                        elif event.button == 3:
                            self.player.gun_equiped = False

                    self.hud.inv.on_click(event.button, self.player)
                    if self.chest.open == False:
                        if mouse_over(pygame.mouse.get_pos(), self.chest.x, self.chest.y, self.chest.rect.width, self.chest.rect.height):
                            self.chest.open = True
                            self.player.opened_chest = True
                    else:
                        self.chest.on_click(event.button, self.hud.inv)

        if self.in_tutorial:  
            self.time += 1
            if self.hud.inventory_open == False and self.chest.open == False:
                self.player.update(self.dt, self.enemies)
            self.enemies.update(self.dt, self.player)

        return False


class Button():
    def __init__(self, color, x, y, width, height, text='', font=40):
        self.color = color
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.text = text
        self.font = font

    def draw(self,win,outline=None):
        if outline:
            pygame.draw.rect(win, outline, (self.x-2,self.y-2,self.width+4,self.height+4),0)
            
        pygame.draw.rect(win, self.color, (self.x,self.y,self.width,self.height),0)
        
        if self.text != '':
            font = pygame.font.Font('assets/font.ttf', self.font)
            text = font.render(self.text, 1, (0,0,0))
            win.blit(text, (self.x + (self.width/2 - text.get_width()/2), self.y + (self.height/2 - text.get_height()/2)))

    def isOver(self, pos):
        if pos[0] > self.x and pos[0] < self.x + self.width:
            if pos[1] > self.y and pos[1] < self.y + self.height:
                return True
            
        return False