import pygame


def draw_text(screen, pos, text, size, color, font_type=None):
    if font_type == None: font_type = "assets/font.ttf"
    font = pygame.font.Font(font_type, size)
    txt_surf = font.render(text, True, color)
    txt_rect = txt_surf.get_rect(topleft=pos)

    screen.blit(txt_surf, txt_rect)


def draw_large_text(surface, text, pos, font, color=pygame.Color('black')):
    words = [word.split(' ') for word in text.splitlines()]  # 2D array where each row is a list of words.
    space = font.size(' ')[0]  # The width of a space.
    max_width, max_height = surface.get_size()
    x, y = pos
    for line in words:
        for word in line:
            word_surface = font.render(word, 0, color)
            word_width, word_height = word_surface.get_size()
            if x + word_width >= max_width:
                x = pos[0]  # Reset the x.
                y += word_height  # Start on new row.
            surface.blit(word_surface, (x, y))
            x += word_width + space
        x = pos[0]  # Reset the x.
        y += word_height  # Start on new row.

def button(win, pos, dimensions, text="", color=(255,0,0), font=10, font_type="assets/font.ttf", image=None):
    if text != "":
        pygame.draw.rect(win, color, (pos[0], pos[1], dimensions[0], dimensions[1]), 0)
        font = pygame.font.Font(font_type, font)
        text = font.render(text, 1, (0,0,0))
        rect = win.blit(text, (pos[0] + (dimensions[0]/2 - text.get_width()/2), pos[1] + (dimensions[1]/2 - text.get_height()/2)))
    elif image != None:
        rect = win.blit(pygame.transform.scale(image, dimensions), pos)
    else:
        rect = None
    return rect


def mouse_over(pos, x, y, width, height):
    if pos[0] > x and pos[0] < x + width:
        if pos[1] > y and pos[1] < y + height:
            return True
        else:
            return False
    else:
        return False


def blurSurf(surface, amt):
    if amt < 1.0:
        raise ValueError("Arg 'amt' must be greater than 1.0, passed in value is %s"%amt)
    scale = 1.0/float(amt)
    surf_size = surface.get_size()
    scale_size = (int(surf_size[0]*scale), int(surf_size[1]*scale))
    surf = pygame.transform.smoothscale(surface, scale_size)
    surf = pygame.transform.smoothscale(surf, surf_size)
    return surf


def image_outline(win, img, pos, color):
    mask = pygame.mask.from_surface(img)
    mask_outline = mask.outline()
    mask_surf = pygame.Surface(img.get_size())
    for pixel in mask_outline:
        mask_surf.set_at(pixel, color)
    mask_surf.set_colorkey((0, 0, 0))
    win.blit(mask_surf,(pos[0]-1, pos[1]))
    win.blit(mask_surf,(pos[0]+1, pos[1]))
    win.blit(mask_surf,(pos[0], pos[1]-1))
    win.blit(mask_surf,(pos[0], pos[1]+1))

    