import pygame
import random
import json
from python.enemies import Enemy
from python.chests import Chest
from python.items import Item


class World():
    def __init__(self, screen_dimensions, chests, enemies, particles, items):
        self.width = screen_dimensions[0]
        self.height = screen_dimensions[1]
        self.scroll = pygame.Vector2(self.width / 2, self.height / 2)
        self.surface = pygame.Surface((100*32, 100*32)).convert_alpha()
        self.tiles = []
        self.extra_tiles = {"pyramids": [], "mountains": [], "volcanos": [], "trees": []}
        self.red_tiles = []
        self.yellow_tiles = []
        self.all_tiles = []
        self.load_tiles()
        #self.generate_world()
        #self.generate_enemies(enemies, particles)
        #self.generate_chests(chests, items)
        #self.render_world()

    def center_camera(self, target):
        self.scroll.x = target.rect.centerx - self.width / 2
        self.scroll.y = target.rect.centery - self.height / 2

    def load_tiles(self):
        #self.red_tile = pygame.transform.scale(pygame.image.load("assets/world/red.png").convert_alpha(), (32,32))
        for x in range(1, 7):
            self.red_tiles.append(pygame.transform.scale(pygame.image.load(f"assets/world/red{x}.png").convert_alpha(), (32,32)))
        for x in range(1, 4):
            self.yellow_tiles.append(pygame.transform.scale(pygame.image.load(f"assets/world/yellow{x}.png").convert_alpha(), (32,32)))
        
        self.mountain = pygame.transform.scale(pygame.image.load("assets/world/mountain.png").convert_alpha(), (64,64))
        self.volcano = pygame.transform.scale(pygame.image.load("assets/world/volcano.png").convert_alpha(), (128, 128))
        self.pyramid = pygame.transform.scale(pygame.image.load("assets/world/pyramid.png").convert_alpha(), (128, 128))
        self.pyramid2 = pygame.transform.scale(pygame.image.load("assets/world/pyramid-with-door.png").convert_alpha(), (128, 128))
        self.tree1 = pygame.transform.scale(pygame.image.load("assets/world/tree1.png").convert_alpha(), (32, 32))

        self.all_tiles = self.red_tiles + self.yellow_tiles

    def update(self, player, enemies, chests, items, particles):
        pass

    def reset_world(self, player, enemies, chests, items, particles):
        player.health = player.max_health
        self.tiles = []
        self.extra_tiles = {"pyramids": [], "mountains": [], "volcanos": [], "trees": []}
        self.generate_world()
        self.generate_enemies(enemies, particles)
        new_chests = self.generate_chests(chests, items)
        self.render_world()
        chests = new_chests
        return chests

    def generate_chests(self, chests, items_):
        chests = []
        items = items_.items
        for x in range(random.randint(2,6)):
            x_ = random.randint(0, 100*32)
            y_ = random.randint(0, 100*32)
            chest = Chest((self.width, self.height), (x_, y_))
            y = 0
            for item in items:
                if items[item]["rarity"] == "common":
                    chance = 25
                    amount = random.randint(1,3)
                elif items[item]["rarity"] == "rare":
                    chance = 10
                    amount = random.randint(1,2)
                elif items[item]["rarity"] == "super rare":
                    chance = 2
                    amount = 1
                r = random.randint(1, 100)
                if chance >= r:
                    chest.add_item(Item(item, items[item], items_.art[item]), amount)
            chests.append(chest)
        return chests

    def generate_enemies(self, enemies, particles):
        for i in range(random.randint(5, 15)):
            enemy_type = random.randint(1,90)
            if enemy_type < 40:
                enemy_type = "normal gnoll"
            elif 40 <= enemy_type <= 50:
                enemy_type = "rare gnoll"
            elif 50 < enemy_type <= 80:
                enemy_type = "mushroom"
            elif 80 < enemy_type <= 100:
                enemy_type = "bounty hunter"
            
            if enemy_type == "normal gnoll":
                enemy_type = random.choice(["gnoll scout", "gnoll brute"])
            elif enemy_type == "rare gnoll":
                enemy_type = random.choice(["gnoll captain", "gnoll shaman"])
            elif enemy_type == "mushroom":
                enemy_type = random.choice(["large mushroom", "normal mushroom", "small mushroom"])
            elif enemy_type == "bounty hunter":
                enemy_type = "bounty hunter"
            else:
                enemy_type = "bounty hunter"

            x = random.randint(0, 100*32)
            y = random.randint(0, 100*32)
            y -= 32
            x -= 32
            enemy = Enemy(x, y, enemy_type, enemies.enemies[enemy_type], enemies.art[enemy_type], particles)
            enemies.enemies_spawned.append(enemy)

    def generate_world(self):
        biomes = ["yellow", "red"]
        current_biome = random.choice(biomes)
        last_tile = None
        for y in range(100):
            for x in range(100):
                if current_biome == "red":
                    list_ = self.red_tiles
                elif current_biome == "yellow":
                    list_ = self.yellow_tiles
                tile = random.choice(list_)
                extra_tile = random.randint(0, 100)
                if current_biome == "red":
                    if extra_tile < 10:
                        extra_tile = self.mountain
                        last_tile = "mountain"
                    elif random.randint(1, 5000) == 1 and last_tile == None:
                        extra_tile = self.volcano
                        last_tile = "volcano"
                    elif 10 < extra_tile < 25:
                        extra_tile = self.tree1
                        last_tile = "tree"
                    else:
                        extra_tile = None
                        last_tile = None
                elif current_biome == "yellow":
                    if extra_tile < 5:
                        extra_tile = self.mountain
                        last_tile = "mountain"
                    elif random.randint(1, 6000) == 1 and last_tile == None:
                        extra_tile = self.pyramid
                        last_tile = "pyramid"
                    elif 10 < extra_tile < 25:
                        extra_tile = self.tree1
                        last_tile = "tree"
                    else:
                        extra_tile = None
                        last_tile = None
                if last_tile == "mountain":
                    self.extra_tiles["mountains"].append((x*32, y*32))
                elif last_tile == "pyramid":
                    self.extra_tiles["pyramids"].append((x*32, y*32))
                elif last_tile == "volcano":
                    self.extra_tiles["volcanos"].append((x*32, y*32))
                elif last_tile == "tree":
                    self.extra_tiles["trees"].append((x*32, y*32))
                tile = {"x": x*32, "y": y*32, "tile": tile, "extra tile": extra_tile}
                self.tiles.append(tile)
    
    def render_world(self):
        for tile in self.tiles:
            x, y = tile["x"], tile["y"]
            self.surface.blit(tile["tile"], (x, y))
            if tile["extra tile"] != None:
                self.surface.blit(tile["extra tile"], (x, y))
        for tree in self.extra_tiles["trees"]:
            self.surface.blit(self.tree1, tree)
        for mountain in self.extra_tiles["mountains"]:
            self.surface.blit(self.mountain, (mountain[0], mountain[1]))
        for volcano in self.extra_tiles["volcanos"]:
            self.surface.blit(self.volcano, (volcano[0], volcano[1]))
        for pyramid in self.extra_tiles["pyramids"]:
            self.surface.blit(self.pyramid2, (pyramid[0], pyramid[1]))
        #for tile in self.tiles:
        #    x, y = tile["x"], tile["y"]
        #    if tile["extra tile"] != None:
        #        self.surface.blit(tile["extra tile"], (x, y))

    def render(self, win):
        win.blit(self.surface, (-self.scroll.x, -self.scroll.y))

    def cart_to_iso(self, x, y):
        iso_x = (x-y)
        iso_y = (x+y)/2
        return iso_x, iso_y


class MiniMap():
    def __init__(self, winsize, world):
        self.world = world
        self.width = winsize[0] 
        self.height = winsize[1] 
        self.surf = pygame.Surface((100,100))
        self.miniplayer = pygame.image.load("assets/player/miniplayer.png").convert_alpha()

    def render(self, win):
        for tile in self.world.tiles:
            x, y = tile["x"], tile["y"]
            self.surf.blit(tile["tile"], (x / 20, y / 20))
        win.blit(self.surf, (self.width - 100, self.height - 100))